const fse = require('fs-extra');
const cheerio = require('cheerio');
const request = require('request');
const path = require('path');
const _ = require('lodash');

const log = require('../libs/log');
const downloader = require('../libs/downloader');


const baseUrl = 'https://www.xvideos.com/';
const baseReqOpts = {
    headers: {
        'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,image/apng,*/*;q=0.8',
        'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_12_6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/61.0.3163.100 Safari/537.36'
    }
};

const crawlMainpage = (config) => {
    crawlMainPageVideos(config, {
        url: baseUrl,
    })
}

const crawlSearch = (config) => {
    crawlVideos(config, {
        baseUrl: baseUrl,
        qs: { k: encodeURI(config.search.trim()) }
    })
}

const crawlMainPageVideos = async(config, reqOpts) => {
    try {
        let page = 0;
        while (true) {
            if (!!config.tag) {
                reqOpts.uri = `/tags/${config.tag}/${page}/`;
            } else {
                if (page > 0) {
                    reqOpts.uri = `/new/${page}/`;
                }
            }
            Object.assign(reqOpts, baseReqOpts);
            const urls = await getVideoUrls(reqOpts);
            if (!urls || urls.length === 0) {
                throw new Error('find nothing!');
            }
            for (const url of urls) {
                const msg = await crawlVideoPage(url, config.downloadDir, config.proxy);
                log.verbose(msg);
                console.log('');
            }
            page += 1;
        }
    } catch (error) {
        console.log(error);
        process.exit(0);
    }
}

const crawlVideos = async(config, reqOpts) => {
    try {
        let page = 0;
        while (true) {
            if (page > 0) {
                reqOpts.qs.p = page;
            }
            Object.assign(reqOpts, baseReqOpts);
            const urls = await getVideoUrls(reqOpts);
            if (!urls || urls.length === 0) {
                throw new Error('find nothing!');
            }
            for (const url of urls) {
                const msg = await crawlVideoPage(url, config.downloadDir, config.proxy);
                log.verbose(msg);
                console.log('');
            }
            page += 1;
        }
    } catch (error) {
        console.log(error);
        process.exit(0);
    }
}

const getVideoUrls = (opts) => {
    return new Promise((resolve, reject) => {
        request(opts, (err, res, body) => {
            if (err) {
                reject(err);
                return;
            }
            const $ = cheerio.load(body);
            const urls = [];
            $('.thumb-under a').each((idx, element) => {
                const url = element.attribs.href;
                if (url.indexOf("/video") === 0 && url.indexOf("THUMBNUM") < 0) {
                    urls.push('https://www.xvideos.com' + url)
                };
            });
            resolve(urls);
        })
    });
}

const crawlVideoPage = (url, downloadDir, proxy) => {
    return new Promise((resolve, reject) => {
        downloadVideo(url).then(
            videoInfo => {
                if (videoInfo) {
                    downloader.download(
                        videoInfo.key,
                        `${videoInfo.title}_${videoInfo.quality}_${videoInfo.key}.mp4`,
                        downloadDir,
                        videoInfo.videoUrl,
                        proxy
                    ).then(
                        (msg) => resolve(msg),
                        (errs) => reject(errs)
                    );
                }
            },
            err => reject(err)
        );
    });
}

const downloadVideo = (url) => {
    let opts = {
        url: url,
    };
    Object.assign(opts, baseReqOpts);
    return new Promise((resolve, reject) => {
        request(opts, (err, res, body) => {
            if (err) {
                reject(err);
            }
            const videoInfo = parseDownloadInfo(body);
            if (videoInfo) {
                // videoInfo.key = key;
                resolve(videoInfo);
            } else {
                reject("FAILED: Can't find any video info.");
            }
        });
    })
}

const parseDownloadInfo = (bodyStr) => {
    const index = bodyStr.indexOf('if (html5player)');
    if (index < 0) {
        return undefined;
    }
    let begin, end;
    let cache = 0;
    for (let i = index; i < bodyStr.length; i++) {
        const tmpStr = bodyStr.substr(i, 1);
        if (tmpStr === '{') {
            if (cache === 0 && !begin) {
                begin = i;
            } else {
                cache++;
            }
        }
        if (tmpStr === '}') {
            if (cache > 0) {
                cache--
            } else {
                end = i;
                break;
            }
        }
    }
    let info = {};
    if (begin >= 0 && end >= 0) {
        let infoStr = bodyStr.substring(begin + 1, end);
        info["title"] = getInfoByName("setVideoTitle", infoStr);
        info["videoUrl"] = getInfoByName("setVideoUrlHigh", infoStr);
        info["quality"] = "high";
        let uri = getInfoByName("setVideoURL", infoStr);
        info["key"] = uri.split('/')[1];
    }

    return info;
};

const getInfoByName = (name, bodyStr) => {
    const index = bodyStr.indexOf(name);
    if (index < 0) {
        return "";
    }
    let begin, end;
    for (let i = index; i < bodyStr.length; i++) {
        const tmpStr = bodyStr.substr(i, 1);
        if (tmpStr === '(') {
            begin = i;
        }
        if (tmpStr === ')') {
            end = i;
            break;
        }
    }
    if (begin >= 0 && end >= 0) {
        console.log(bodyStr.substring(begin + 2, end - 1))
        return bodyStr.substring(begin + 2, end - 1);
    }
    return "";
}

const findTitle = (bodyStr) => {
    const $ = cheerio.load(bodyStr);
    const title = $('title').text();
    const arr = title.split('-');
    arr.pop();
    return arr.join('-').trim();
};

const start = (config) => {
    if (config.proxy !== undefined) {
        baseReqOpts.proxy = config.proxy;
    }
    if (!fse.existsSync(config.downloadDir)) {
        fse.mkdirpSync(config.downloadDir);
    }
    if (config.key !== undefined) {
        crawlVideoPage(config.key, config.downloadDir, config.proxy).then(
            msg => {
                log.verbose(msg);
                process.exit(0);
            },
            errs => {
                log.error(errs);
                process.exit(0);
            }
        );
    } else if (config.search !== undefined) {
        crawlSearch(config);
    } else {
        crawlMainpage(config);
    }
};

module.exports = {
    start
};